#include <xcb.h>

#define WINDOW_CLOSED 16
#define CONNECTION_LOST 17
#define CONNECTION_FAILED 18
#define FONT_ERROR 19
#define GRAPHICS_ERROR 20

struct connection {
	xcb_connection_t *c;
	xcb_screen_t *screen;
	const xcb_setup_t *setup;
	xcb_keysym_t *keysyms;
	xcb_get_keyboard_mapping_reply_t *gkmr;
	xcb_font_t font;
	xcb_atom_t wm_protocols, wm_delete_window;
};

#define BUTTON_HOVER 1
#define BUTTON_HELD 2
#define BUTTON_FOCUSED 4
#define TEXTBOX_FOCUSED 8
#define TEXTBOX_HELD 16

struct window {
	struct connection *c;
	xcb_window_t w;
	xcb_gcontext_t bog, wog, bow, gow;
	int flags;
};

int init_connection(struct connection *c);
int create_color(struct connection *c, int red, int green, int blue,
	int backup);
void destroy_connection(struct connection *c);

int create_window(struct connection *c, struct window *w);
void set_normal_hint_size(struct window *w, int width, int height);
xcb_keysym_t get_keysym(struct window *w, xcb_key_press_event_t *kpe);
int create_gc(struct window *w, int fg, int bg, xcb_gcontext_t *gc);
int measure_text(struct window *w, char *s, size_t len, int *width);
int char_at_width(struct window *w, char *s, int width, size_t *length);
int render_linear_text(struct window *w, char *s, size_t len, int x, int y,
	xcb_gcontext_t gc);
int measure_multiline(struct window *w, char *s, int width, int *lines);
int render_multiline(struct window *w, char *s, int x, int y, int width,
	xcb_gcontext_t gc);
void draw_button(struct window *w, xcb_rectangle_t *rect);
void destroy_window(struct window *w);

int message_window(struct connection *c, char *message);
